local M = {}

local destinationMarker = nil --holds dest info, used for gps

local function createMissionGroup(missionGroupName)
  if scenetree[missionGroupName] then
    return
  end --already exists
  if not scenetree.MissionGroup then
    log("E", "createMissionGroup", "scenetree.MissionGroup does not exist!")
    return
  end

  local simGroupObject = createObject("SimGroup")
  simGroupObject:registerObject(missionGroupName)
  simGroupObject.canSave = false

  scenetree.MissionGroup:addObject(simGroupObject.obj)
  log("I", "createMissionGroup", "Created SimGroup " .. missionGroupName)
end

local function deleteMissionGroup(missionGroupName)
  if not scenetree[missionGroupName] then
    return
  end --doesn't exist
  scenetree[missionGroupName]:delete()
end

local function createIconModel(name, groupName, position, color)
  print("Creating icon for:", name, groupName)
  local iconModel = createObject("TSStatic")
  position.z = position.z + 5
  iconModel.shapeName = "art/shapes/interface/" .. groupName .. ".dae"
  iconModel:setPosition(position)
  iconModel.scale = Point3F(1, 1, 1)
  iconModel.useInstanceRenderData = 1
  iconModel.instanceColor = color
  iconModel:registerObject(name .. "_marker_icon")

  return iconModel
end

-- takes string, string, point3f, point4f
local function createScenarioMarker(name, groupName, position, color)
  local marker = createObject("TSStatic")
  marker.shapeName = "art/shapes/interface/checkpoint_marker_base.dae"
  marker:setPosition(position)
  marker.scale = Point3F(2, 2, 2)
  marker.useInstanceRenderData = 1
  marker.instanceColor = color

  marker:registerObject(name .. "_marker")

  ---

  if groupName == "contact" then
    createMissionGroup("CarpContactTriggersGroup")

    scenetree.CarpContactTriggersGroup:addObject(marker.obj)

    local contactLocationTrigger = createObject("BeamNGTrigger")
    contactLocationTrigger.loadMode = 1 --'Manual'
    contactLocationTrigger:setField("triggerType", 0, "Sphere")
    contactLocationTrigger:setField("triggerMode", 0, "Contains")
    contactLocationTrigger.ticking = true
    contactLocationTrigger.tickPeriod = 2000
    contactLocationTrigger.luaFunction = String("carp.receiveContactInfo")
    contactLocationTrigger:setPosition(position)
    contactLocationTrigger:setScale(Point3F(2, 2, 2))
    contactLocationTrigger:registerObject(groupName .. "_" .. name .. "_contactLocationTrigger")
    scenetree.CarpContactTriggersGroup:addObject(contactLocationTrigger.obj)

    local icon = createIconModel(name, groupName, position, color)
    scenetree.CarpContactTriggersGroup:addObject(icon.obj)
  elseif groupName == "gasstation" then
    createMissionGroup("CarpGasStationTriggersGroup")
    scenetree.CarpGasStationTriggersGroup:addObject(marker.obj)

    local gasStationTrigger = createObject("BeamNGTrigger")
    gasStationTrigger.loadMode = 1 --'Manual'
    gasStationTrigger:setField("triggerType", 0, "Sphere")
    gasStationTrigger:setField("triggerMode", 0, "Contains")
    gasStationTrigger.ticking = true
    gasStationTrigger.tickPeriod = 1800
    gasStationTrigger.luaFunction = String("carp.showGasStationPopup")
    gasStationTrigger:setPosition(position)
    gasStationTrigger:setScale(Point3F(2, 2, 2))
    gasStationTrigger:registerObject(groupName .. "_" .. name .. "_gasStationTrigger")

    local icon = createIconModel(name, groupName, position, color)
    scenetree.CarpGasStationTriggersGroup:addObject(icon.obj)
  elseif groupName == "repairgarage" then
    -- marker
    createMissionGroup("CarpCarRepairTriggersGroup")
    marker.scale = Point3F(5, 5, 2)
    scenetree.CarpCarRepairTriggersGroup:addObject(marker.obj)

    -- trigger
    local garageLocationTrigger = createObject("BeamNGTrigger")
    garageLocationTrigger.loadMode = 1 --'Manual'
    garageLocationTrigger:setField("triggerType", 0, "Sphere")
    garageLocationTrigger:setField("triggerMode", 0, "Contains")
    garageLocationTrigger.ticking = true
    garageLocationTrigger.tickPeriod = 2000
    garageLocationTrigger.luaFunction = String("carp.showGaragePopup")
    garageLocationTrigger:setPosition(position)
    garageLocationTrigger:setScale(Point3F(5, 5, 2))
    garageLocationTrigger:registerObject(groupName .. "_" .. name .. "_garageLocationTrigger")
    scenetree.CarpCarRepairTriggersGroup:addObject(garageLocationTrigger)
    scenetree.CarpCarRepairTriggersGroup:addObject(garageLocationTrigger.obj)

    -- icon
    local icon = createIconModel(name, groupName, position, color)
    scenetree.CarpCarRepairTriggersGroup:addObject(icon.obj)
  elseif groupName == "cardealership" then
    createMissionGroup("CarpCarDealershipTriggersGroup")

    marker.scale = Point3F(3, 3, 2)
    scenetree.CarpCarDealershipTriggersGroup:addObject(marker.obj)

    -- create UI trigger:
    local dealershipTrigger = createObject("BeamNGTrigger")
    dealershipTrigger.loadMode = 1
    dealershipTrigger:setField("triggerType", 0, "Sphere")
    dealershipTrigger.luaFunction =
      String(
      "local function triggerhandler(data) if MPVehicleGE.isOwn(data.subjectID) then TriggerServerEvent('requestOwnedVehicles', 'owo') imguipls.showDealershipWindow(data.event=='enter') end end return triggerhandler"
    )
    dealershipTrigger:setPosition(position)
    dealershipTrigger:setScale(Point3F(3, 3, 3))
    dealershipTrigger:registerObject(groupName .. "_" .. name .. "_dealershipTrigger")
    scenetree.CarpCarDealershipTriggersGroup:addObject(dealershipTrigger)
    scenetree.CarpCarDealershipTriggersGroup:addObject(dealershipTrigger.obj)

    -- add icon
    local icon = createIconModel(name, groupName, position, color)
    scenetree.CarpCarDealershipTriggersGroup:addObject(icon.obj)
  elseif groupName == "foodlocation" then
    createMissionGroup("CarpFoodLocationTriggersGroup")

    marker.scale = Point3F(3, 3, 2)
    scenetree.CarpFoodLocationTriggersGroup:addObject(marker.obj)

    local foodLocationTrigger = createObject("BeamNGTrigger")
    foodLocationTrigger.loadMode = 1 --'Manual'
    foodLocationTrigger:setField("triggerType", 0, "Sphere")
    foodLocationTrigger:setField("triggerMode", 0, "Contains")
    foodLocationTrigger.ticking = true
    foodLocationTrigger.tickPeriod = 2000
    foodLocationTrigger.luaFunction = String("carp.receiveFood")
    foodLocationTrigger:setPosition(position)
    foodLocationTrigger:setScale(Point3F(3, 3, 3))
    foodLocationTrigger:registerObject(groupName .. "_" .. name .. "_foodLocationTrigger")

    -- add icon
    local icon = createIconModel(name, groupName, position, color)
    scenetree.CarpCarDealershipTriggersGroup:addObject(icon.obj)
  elseif groupName == "debug" then
    createMissionGroup("CarpMissionDebugGroup")
    marker.scale = Point3F(2, 2, 2)
    scenetree.CarpMissionDebugGroup:addObject(marker.obj)
  else
    createMissionGroup("CarpMissionTriggersGroup")

    local icon = createIconModel(name, groupName, position, color)
    scenetree.CarpMissionTriggersGroup:addObject(marker.obj)
    scenetree.CarpMissionTriggersGroup:addObject(icon.obj)

    local missionTrigger = createObject("BeamNGTrigger")
    missionTrigger.loadMode = 1
    missionTrigger:setField("triggerType", 0, "Sphere")
    missionTrigger.luaFunction =
      String(
      "local function triggerhandler(data) " ..
        " if MPVehicleGE.isOwn(data.subjectID) then " ..
          " carp.ePress(data.event=='enter') " ..
            " Engine.Audio.playOnce('AudioMaster', 'audio/you_have_arrived.ogg') end " .. "end return triggerhandler"
    )
    missionTrigger:setPosition(position)
    missionTrigger:setScale(Point3F(1.2, 1.2, 1.2))
    missionTrigger:registerObject(groupName .. "_" .. name .. "_missionTrigger")
    scenetree.CarpMissionTriggersGroup:addObject(missionTrigger)
    scenetree.CarpMissionTriggersGroup:addObject(missionTrigger.obj)
  end
  return marker
end

local function carpSpawnWaypoint(name, pos)
  local coords = stringToTable(pos, "|")
  local p3fCoords = Point3F(tonumber(coords[1]), tonumber(coords[2]), tonumber(coords[3]) + 0.6)

  --local colorArray = stringToTable(coords[4], ",")
  local color = Point4F(1, 1, 1, 1)

  local pointType = stringToTable(name, "-")[1]

  if pointType == "contact" then
    color = Point4F(1, 0.6, 0, 1)
  elseif pointType == "gasstation" then
    color = Point4F(0.6, 0.5, 1, 1)
  elseif pointType == "repairgarage" then
    color = Point4F(0.1, 1, 0.3, 1)
  elseif pointType == "cardealership" then
    color = Point4F(0.85, 0.2, 0.85, 1)
  elseif pointType == "foodlocation" then
    color = Point4F(0.1, 0.1, 1, 1)
  else
    color = Point4F(1, 0, 0, 1)
  end
  return createScenarioMarker(name, pointType, p3fCoords, color), p3fCoords
end

local function onClientSpawnMarker(missionData)
  local nameAndPos = stringToTable(missionData, ";")
  local name = nameAndPos[1]
  local pos = nameAndPos[2]
  carpSpawnWaypoint(name, pos)
  print("spawned waypoint called " .. name)
end

local function onClientStartMission(missionData)
  local decodedData = jsonDecode(missionData)

  core_groundMarkers.resetAll()
  core_groundMarkers.sendToApp()

  local name = decodedData.missionDestination
  local pos = decodedData.destinationCoords

  name = name:gsub("contact", "mission") -- make sure contact markers behave as destination ones

  if (string.find(name, "mission") == nil) then
    name = "mission-" .. name
  end

  deleteMissionGroup("CarpContactTriggersGroup")

  destinationMarker, pos = carpSpawnWaypoint(name, pos)
  print("spawned waypoint called " .. name)

  MPVehicleGE.queryRoadNodeToPosition(vec3(pos), "carpTarget")
  carp.syncWaypoint()
end

local function onClientEndMission(decodedData)
  if scenetree.CarpMissionTriggersGroup then
    scenetree.CarpMissionTriggersGroup:delete()
  end
  core_groundMarkers.resetAll()
  core_groundMarkers.sendToApp()
end

local function removeDebugWaypoints()
  if scenetree.CarpMissionDebugGroup then
    scenetree.CarpMissionDebugGroup:delete()
  end
end

local function onExtensionLoaded()
  if MPCoreNetwork then
    AddEventHandler("onClientStartMission", onClientStartMission)
    AddEventHandler("onClientEndMission", onClientEndMission)

    AddEventHandler("onClientSpawnMarker", onClientSpawnMarker)

    AddEventHandler("removeDebugWaypoints", removeDebugWaypoints)
    --AddEventHandler("onClientRemoveMissionMarkers", onClientRemoveMissionMarkers) not implemented

    print("[CaRP Markers <><] Ready!")
  end
end

M.onExtensionLoaded = onExtensionLoaded

return M
