--====================================================================================
-- All work by Pastry.
-- You have no permission to edit, redistribute or upload. That's all!
--====================================================================================
-- CaRP UI
--====================================================================================
local M = {}
print("Loading Carp Generic UI...")

M.dependencies = { "ui_imgui" }

local gui_module = require("ge/extensions/editor/api/gui")
local gui = { setupEditorGuiTheme = nop }
local im = ui_imgui

local defaultSettings = {
  window_scale = 1,
  icon_scale = 0.8,
  sfx_volume = 5,
  mission_auto_complete = true,
  distance_lights_distance = 50,
  distance_lights_enabled = true,
  rain_enabled = true,
  disable_groundmarkers = false,
  garage_mode_enabled = false
}
local UI_SETTINGS = nil

local UI_MISSION_DESCRIPTION = nil
local UI_BANK_BALANCE = 0

local UI_EARNED = 0.0
local UI_SPENT = 0.0
local UI_MONEY_PER_MINUTE = nil

local availableDrawItems = {}

isThemePushed = false

local carpWindows = {
  CarpCustomUI = "Custom window",
  CarpSpawningUI = "Spawn window",
  CarpHelpUI = "Help window",
  CarpPartyUI = "Party window",
  CarpSettingsUI = "Settings window",
  CarpWeatherUI = "Weather window",
  CarpDailyMissionUI = 'Daily Bonuses window'
}

--theme style
function setWindowTheme()
  if not isThemePushed then
    Engine.imgui.enableBeamNGStyle()
    log("I", "", "Pushing BeamNG Style")
  end

  if not isThemePushed then
    log("I", "", "Pushing CaRP Style")
    --push style
    im.PushStyleColor2(im.Col_Button, im.ImVec4(0.2, 0.2, 0.2, 0.7))
    im.PushStyleColor2(im.Col_ButtonHovered, im.ImVec4(0.3, 0.3, 0.3, 0.7))
    im.PushStyleColor2(im.Col_ButtonActive, im.ImVec4(0.15, 0.35, 0.8, 0.7))
    im.PushStyleColor2(im.Col_Text, im.ImVec4(1, 1, 1, 1))
    im.PushStyleColor2(im.Col_TextDisabled, im.ImVec4(0.8, 0.8, 0.8, 0.5))
    im.PushStyleColor2(im.Col_WindowBg, im.ImVec4(0, 0, 0, 0.9))
    im.PushStyleColor2(im.Col_PopupBg, im.ImVec4(0, 0, 0, 0.9))
    im.PushStyleColor2(im.Col_FrameBg, im.ImVec4(0, 0, 0, 0.8))
    im.PushStyleColor2(im.Col_TitleBg, im.ImVec4(0, 0, 0, 0.9))
    im.PushStyleColor2(im.Col_TitleBgActive, im.ImVec4(0.15, 0.35, 0.8, 0.9))
    im.PushStyleColor2(im.Col_Header, im.ImVec4(0.2, 0.2, 0.2, 0.7))
    im.PushStyleColor2(im.Col_HeaderHovered, im.ImVec4(0.3, 0.3, 0.3, 0.7))
    im.PushStyleColor2(im.Col_HeaderActive, im.ImVec4(0.15, 0.35, 0.8, 0.9))

    im.PushStyleVar1(im.StyleVar_WindowRounding, 0.0)
    im.PushStyleVar1(im.StyleVar_ChildRounding, 0.0)
    im.PushStyleVar1(im.StyleVar_PopupRounding, 0.0)
    im.PushStyleVar1(im.StyleVar_FrameRounding, 0.0)
    im.PushStyleVar1(im.StyleVar_ScrollbarRounding, 0.0)
    im.PushStyleVar1(im.StyleVar_GrabRounding, 0.0)
    im.PushStyleVar1(im.StyleVar_TabRounding, 0.0)

    isThemePushed = true
  end
end

local function loadSavedSettings()
  UI_SETTINGS = jsonReadFile("settings/carpsettings.json")

  if not UI_SETTINGS then
    UI_SETTINGS = jsonReadFile("carpsettings.json")
    if UI_SETTINGS then -- migrate old settings file
      jsonWriteFile("settings/carpsettings.json", UI_SETTINGS)
      FS:removeFile("carpsettings.json") -- dispose of the evidence
    else
      UI_SETTINGS = defaultSettings
    end
  end
end

local function saveUISetting(settingName, value)
  UI_SETTINGS[settingName] = value
  jsonWriteFile("settings/carpsettings.json", UI_SETTINGS)
end

local function getUISetting(settingName)
  if UI_SETTINGS[settingName] == nil then
    return defaultSettings[settingName]
  end

  return UI_SETTINGS[settingName]
end

-- draw bits
local function drawWindowOptions()
  if im.CollapsingHeader1("Window options") then
    if im.Button("Larger Text") then
      saveUISetting("window_scale", getUISetting("window_scale") + 0.05)
    end
    if im.Button("Smaller Text") then
      saveUISetting("window_scale", getUISetting("window_scale") - 0.05)
    end
    im.Separator()

    if im.Button("Larger Icons") then
      saveUISetting("icon_scale", getUISetting("icon_scale") + 0.05)
    end
    if im.Button("Smaller Icons") then
      saveUISetting("icon_scale", getUISetting("icon_scale") - 0.05)
    end
  end
end

local function drawBalance()
  im.TextWrapped("Your balance is: $" .. UI_BANK_BALANCE .. ".")

  local earningsText = "This session, you've earned $" .. UI_EARNED .. "."

  if UI_MONEY_PER_MINUTE then
    local formattedMoneyPerMinute = string.format("%.2f", UI_MONEY_PER_MINUTE)

    earningsText = earningsText .. " ($" .. formattedMoneyPerMinute .. "/minute)"
  end

  im.TextWrapped(earningsText)

  im.TextWrapped("This session, you've spent $" .. UI_SPENT .. ".")
end

local function drawShowHideMenu()
  if im.CollapsingHeader1("Show/Hide windows") then
    local icon_size = M.getIconSize()

    for windowName, windowTitle in pairs(carpWindows) do
      if M.isWindowVisible(windowName) then
        if M.uiIconImageButton(M.icons.visibility, icon_size) then
          M.hideWindow(windowName)
          M.saveUISetting("hide_window_" .. windowName, true)
        end
        im.SameLine()
        im.TextWrapped("Hide " .. windowTitle)
      else
        if M.uiIconImageButton(M.icons.visibility_off, icon_size) then
          M.showWindow(windowName)
          M.saveUISetting("hide_window_" .. windowName, false)
        end
        im.SameLine()
        im.TextWrapped("Show " .. windowTitle)
      end
    end
  end
  --im.EndChild()
end

-- event handlers: -->
local function onClientStartMission(missionData)
  local decodedData = jsonDecode(missionData)

  if (decodedData.missionMessage and decodedData.missionMessage ~= "") then
    UI_MISSION_DESCRIPTION = decodedData.missionMessage
    UI_MISSION_DESCRIPTION = UI_MISSION_DESCRIPTION:gsub("%%", "%%%%")
  end

  if (decodedData.special ~= nil and UI_MISSION_DESCRIPTION ~= nil) then
    UI_MISSION_DESCRIPTION = "Trailer Mission: " .. tostring(UI_MISSION_DESCRIPTION)
  end
end

local function onClientEndMission()
  UI_MISSION_DESCRIPTION = nil
end

local function onClientReceiveBalance(data)
  UI_BANK_BALANCE = tonumber(data)
end

local function onClientReceiveProfitStats(data)
  local decodedData = jsonDecode(data)

  if not decodedData or not decodedData.totalEarned or not decodedData.totalSpent then
    return
  end

  UI_EARNED = tonumber(decodedData.totalEarned)
  UI_SPENT = tonumber(decodedData.totalSpent)

  if decodedData.moneyPerMinute then
    UI_MONEY_PER_MINUTE = decodedData.moneyPerMinute
  end
end

local function onExtensionLoaded()
  M.guiModule = require("editor/api/gui")

  M.guiModule.initialize(M)

  loadSavedSettings()

  if MPCoreNetwork then
    AddEventHandler("onClientStartMission", onClientStartMission)
    AddEventHandler("onClientEndMission", onClientEndMission)
    AddEventHandler("onClientReceiveBalance", onClientReceiveBalance)
    AddEventHandler("onClientReceiveProfitStats", onClientReceiveProfitStats)
  end

  log("I", "onExtensionLoaded", "CaRP Base UI initialized.")

  M.registerDrawItem("balance", "Balance", drawBalance)
  M.registerDrawItem("windowOptions", "UI Options", drawWindowOptions)
  M.registerDrawItem("hideWindows", "Show/Hide windows", drawShowHideMenu)
end

local function onEditorToolWindowHide(name)
  if name:startswith("Carp") then
    saveUISetting("hide_window_" .. name, true)
  end
end

local function onUpdate()
  setWindowTheme()
  --drawCommandMenu()
end

local function textWithFont(text, fontName, isWrapped)
  im.PushFont3(fontName)

  if isWrapped then
    im.TextWrapped(text)
  else
    im.Text(text)
  end

  im.PopFont()
end

local function getBalance()
  return UI_BANK_BALANCE
end

local function getMissionDescription()
  return UI_MISSION_DESCRIPTION
end

local function getIconSize()
  local px = 32 * getUISetting("icon_scale") * getUISetting("window_scale")
  return im.ImVec2(px, px)
end

local function getAvailableDrawItems()
  return availableDrawItems
end

local function registerDrawItem(name, title, func, isVisible)
  availableDrawItems[name] = {
    title = title,
    draw = func,
    isVisible = isVisible or function()
      return true
    end
  }
end

M.getUISetting = getUISetting
M.saveUISetting = saveUISetting

M.getBalance = getBalance
M.getMissionDescription = getMissionDescription
M.getIconSize = getIconSize
M.getAvailableDrawItems = getAvailableDrawItems
M.registerDrawItem = registerDrawItem

M.drawWindowOptions = drawWindowOptions
M.drawShowHideMenu = drawShowHideMenu
M.textWithFont = textWithFont
M.drawBalance = drawBalance

M.onUpdate = onUpdate
M.onExtensionLoaded = onExtensionLoaded

M.onEditorToolWindowHide = onEditorToolWindowHide

print("Generic UI loaded")
return M
