--====================================================================================
-- All work by Pastry and deer.
-- You have no permission to edit, redistribute or upload. That's all!
--====================================================================================
-- CaRP UI
--====================================================================================
local M = {}
print("Loading Carp Dealership UI...")

M.dependencies = {"ui_imgui", "carp_ui_base"}

local im = ui_imgui
local base = carp_ui_base

local dealershipName = nil
local groupedVehicles = {}

local colors = {
  bought = im.ImColorByRGB(26, 240, 119, 255),
  buy = im.ImColorByRGB(255, 192, 18, 255),
  locked = im.ImColorByRGB(247, 54, 54, 255)
}
local headingFont = "cairo_semibold_large"

--================ Vehicle Drawing Functions ================--
local function drawLockedVehicle(vehicle)
  base.uiIconImageButton(base.icons.lock, nil, colors.locked.Value)

  if im.IsItemHovered() then
    im.SetTooltip(
      "This vehicle is locked. Purchase at least one vehicle in all levels below in the same class to unlock this one."
    )
  end

  im.SameLine()

  im.Text("$" .. vehicle.price) -- im.Text("Locked")

  if im.IsItemHovered() then
    im.SetTooltip(
      "This vehicle is locked. Purchase at least one vehicle in all levels below in the same class to unlock this one."
    )
  end
end

local function drawPurchasableVehicle(vehicle)
  local pressed = base.uiIconImageButton(base.icons.shopping_cart, nil, colors.buy.Value)

  im.SameLine()
  im.Text("$" .. vehicle.price)

  if pressed then
    UI.chatSend("/buy " .. vehicle.jbeamName)
  end
end

local function drawOwnedVehicle(vehicle)
  base.uiIconImageButton(base.icons.check or base.icons.done, nil, colors.bought.Value)
  im.SameLine()

  if vehicle.isProLicensed then
    im.Text("Prolicensed")
  else
    if im.Button("Prolicense $" .. vehicle.prolicensePrice) then
      UI.chatSend("/prolicense " .. vehicle.jbeamName)
    end
  end
end

local function drawVehicleItem(vehicle)
  im.BeginGroup()

  im.Text(vehicle.name)

  if vehicle.isOwned then
    drawOwnedVehicle(vehicle)
  elseif vehicle.canBeBought then
    drawPurchasableVehicle(vehicle)
  else
    drawLockedVehicle(vehicle)
  end

  im.EndGroup()

  --if _ < #vehicleGroup then im.Separator() end -- wish this worked as intended
end

local function drawVehicleGroup(vehicleGroup)
  im.BeginGroup()

  for _, vehicle in ipairs(vehicleGroup) do
    drawVehicleItem(vehicle)
  end

  im.EndGroup()
end
--============ END Vehicle Drawing Functions END ============--

--=============== Dealership Layout Functions ===============--
local function drawTierText(tierString)
  for i = 1, groupedVehicles.maxTier do
    base.textWithFont(tierString .. tostring(i), headingFont)

    im.NextColumn()
  end
end

local function drawDealershipRow(vehicleCategoryRow)
  for groupid, vehGroup in ipairs(vehicleCategoryRow) do
    if vehGroup.level > 0 then -- skip pico for now so its not in the corner
      drawVehicleGroup(vehGroup)
      im.NextColumn()
    end
  end
end

local function buildDealershipLayout()
  im.Columns(groupedVehicles.maxTier + 1, "dealershipList") -- +1 because pico is 0
  im.NextColumn() -- skip pico

  -- Print tier text
  drawTierText("A")
  im.NextColumn() -- skip pico

  --im.Text('A') im.NextColumn()
  drawDealershipRow(groupedVehicles.a)

  im.Separator()
  im.Separator()

  --im.Text('B') im.NextColumn()
  drawVehicleGroup(groupedVehicles.a[1]) -- pico
  im.NextColumn()

  drawDealershipRow(groupedVehicles.b)

  -- if there are more A vehicles than B (currently the case)
  if not groupedVehicles.b[groupedVehicles.maxTier] then
    im.NextColumn()
  end

  im.NextColumn() -- skip pico
  -- Print tier text
  drawTierText("B")

  im.Columns(1, "dealershipList")
end

local function drawDealershipUI()
  if not base.beginWindow("CarpDealershipUI", "CaRP Dealership", nil, true) then base.endWindow() return end

  im.SetWindowFontScale(base.getUISetting("window_scale"))
  base.textWithFont("Welcome to " .. (dealershipName or "[insert funny dealership name]") .. "!", headingFont, true)

  -- still loading, show loading message:
  if not groupedVehicles.maxTier then
    base.textWithFont("Loading data...", headingFont, true)
    return
  end

  buildDealershipLayout()

  base.endWindow()
end
--=========== END Dealership Layout Functions END ===========--

local function showWindow(show, name)
  if show then
    dealershipName = name
    base.showWindow("CarpDealershipUI")
  else
    base.hideWindow("CarpDealershipUI")
  end
end

local function onReceiveDealershipData(data)
  local keyValuePair = jsonDecode(data)
  local vehicleArray = {}

  -- turn incoming data sortable
  local maxTier = 1
  for k, v in pairs(keyValuePair) do
    v.jbeamName = k
    if v.tier.level > maxTier then
      maxTier = v.tier.level
    end
    table.insert(vehicleArray, v)
  end

  -- sort vehicles
  table.sort(
    vehicleArray,
    function(v1, v2)
      return v1.tier.level < v2.tier.level
    end
  ) -- levels decreasing

  local a, b = {}, {}
  for k, v in ipairs(vehicleArray) do
    table.insert(v.tier.class == "A" and a or b, v)
  end -- separate tiers

  groupedVehicles = {maxTier = maxTier, a = {}, b = {}}

  for classname, classcontents in pairs {a = a, b = b} do
    local currentgroupname = ""
    local currentgroupid = 0

    for k, v in ipairs(classcontents) do
      local groupName = v.tier.class .. v.tier.level
      if currentgroupname ~= groupName then
        currentgroupid = currentgroupid + 1
        groupedVehicles[classname][currentgroupid] = {name = groupName, class = v.tier.class, level = v.tier.level}
        currentgroupname = groupName
      end

      table.insert(groupedVehicles[classname][currentgroupid], v)
    end
  end

  --do
  --  local currentgroupname = ''
  --  local currentgroupid = 0
  --
  --  for k, v in ipairs(a) do
  --    local groupName = v.tier.class .. v.tier.level
  --    if currentgroupname ~= groupName then
  --      currentgroupid = currentgroupid + 1
  --      groupedVehicles.a[currentgroupid] = {name=groupName, class=v.tier.class, level=v.tier.level}
  --      currentgroupname = groupName
  --    end
  --
  --    table.insert(groupedVehicles.a[currentgroupid], v)
  --  end
  --end
  --
  --do
  --  local currentgroupname = ''
  --  local currentgroupid = 0
  --
  --  for k, v in ipairs(b) do
  --    local groupName = v.tier.class .. v.tier.level
  --    if currentgroupname ~= groupName then
  --      currentgroupid = currentgroupid + 1
  --      groupedVehicles.b[currentgroupid] = {name=groupName, class=v.tier.class, level=v.tier.level}
  --      currentgroupname = groupName
  --    end
  --
  --    table.insert(groupedVehicles.b[currentgroupid], v)
  --  end
  --end

  --print('-------------group names:')
  --for k,v in ipairs(groupedVehicles) do print(v.name) end
  --print('-------------dump:')
  --dump(groupedVehicles)
end

local function onExtensionLoaded()
  if MPCoreNetwork then
    base.registerWindow("CarpDealershipUI", im.ImVec2(450, 300)) -- dealership stuff, shows up at a car dealer

    AddEventHandler("onReceiveDealershipData", onReceiveDealershipData)

    print("Dealership UI initialized.")
  end
end

local function onUpdate()
  drawDealershipUI()
end

M.onExtensionLoaded = onExtensionLoaded
M.onUpdate = onUpdate
M.showWindow = showWindow

print("Carp UI loaded")
return M
