--====================================================================================
-- All work by Pastry and deer.
-- You have no permission to edit, redistribute or upload. That's all!
--====================================================================================
-- CaRP spawn UI
--====================================================================================
local M = {}
print("Loading Carp Garage UI...")

M.dependencies = { "ui_imgui", "carp_ui_base" }

local im = ui_imgui
local base = carp_ui_base

local slotKeys = { "configSlotOne", "configSlotTwo", "configSlotThree" }
local slotNames = {}
local UI_GARAGE_NAME = nil
local editingSlotID = nil
local buf_size = 64
local slotnamebuf = im.ArrayChar(buf_size)

local headingFont = "cairo_semibold_large"

-- not pretty, i know
-- TODO: move this to the DB
local function loadSlotNames()
  slotNames = {
    base.getUISetting('configSlotName_one'),
    base.getUISetting('configSlotName_two'),
    base.getUISetting('configSlotName_three')
  }
end

local function saveSlotNames()
  base.saveUISetting('configSlotName_one', slotNames[1])
  base.saveUISetting('configSlotName_two', slotNames[2])
  base.saveUISetting('configSlotName_three', slotNames[3])
end

-- draw functions: -->
local function drawGaragePopup()
  if not base.beginWindow("GarageUI", "Garage!", nil, true) then base.endWindow() return end

  im.SetWindowFontScale(base.getUISetting("window_scale"))

  if UI_GARAGE_NAME then
    base.textWithFont("Welcome to " .. UI_GARAGE_NAME .. "!", headingFont, true)
    im.TextWrapped("You are now in a garage.")
  else
    im.TextWrapped("Welcome! You are now in a garage.")
  end

  im.TextWrapped("Costs for car repairs and installing/uninstalling parts are reduced while you're here.")

  im.TextWrapped("Press Ctrl+W to buy and install parts on your vehicle.")

  im.Spacing()

  im.TextWrapped("Don't see prices for parts? Click here:")
  if im.Button("Reload UI") then
    reloadUI()
  end

  base.endWindow()
end

local function drawSpawnLastUsedConfig()
  local icon_size = base.getIconSize()
  im.TextWrapped("Spawn your last used config:")
  im.SameLine(im.GetContentRegionAvailWidth() - icon_size.x) -- right align
  if base.uiIconImageButton(base.icons.fg_vehicle_sports_car, icon_size) then
    TriggerServerEvent("receiveLastLocationAndVehicle", "a")
  end
end

local function drawSpawnCustomConfigs()
  local icon_size = base.getIconSize()
  for index, key in pairs(slotKeys) do
    local name = slotNames[index] or "Custom Config Slot " .. index

    if editingSlotID == index then
      im.InputText("##carpcustomvehiclename", slotnamebuf, buf_size)
      im.SameLine()
      if base.uiIconImageButton(base.icons.cancel, icon_size) then
        editingSlotID = nil
      end
      im.SameLine()
      if base.uiIconImageButton(base.icons.save, icon_size) then
        editingSlotID = nil
        slotNames[index] = ffi.string(slotnamebuf)
        saveSlotNames()
      end

    else
      im.TextWrapped(name .. ":")
      im.SameLine()
      if base.uiIconImageButton(base.icons.mode_edit, icon_size) then
        slotnamebuf = im.ArrayChar(buf_size, slotNames[index] or "")
        editingSlotID = index
      end
    end

    im.SameLine(im.GetContentRegionAvailWidth() - (icon_size.x + im.GetStyle().ItemSpacing.x) * 2) -- right align

    if base.uiIconImageButton(base.icons.save, icon_size) then
      TriggerServerEvent("saveLatestConfigToSlot", key)
    end
    if im.IsItemHovered() then
      im.SetTooltip("Save current vehicle as slot '" .. name .. "'")
    end

    im.SameLine(im.GetContentRegionAvailWidth() - icon_size.x) -- right align

    if base.uiIconImageButton(base.icons.fg_vehicle_sports_car, icon_size) then
      TriggerServerEvent("receiveSpawnSavedConfig", key)
    end
    if im.IsItemHovered() then
      im.SetTooltip("Summon (spawn) vehicle '" .. name .. "'")
    end

    im.Separator()
    im.Spacing()
  end
end

local function drawSpawningUI()
  if not base.beginWindow("CarpSpawningUI", "CaRP Spawn Menu") then base.endWindow() return end

  im.SetWindowFontScale(base.getUISetting("window_scale"))
  local icon_size = base.getIconSize()

  -- new picoturdie
  im.TextWrapped("Spawn a Pico Turdie (starter vehicle):")
  im.SameLine(im.GetContentRegionAvailWidth() - icon_size.x) -- right align
  if base.uiIconImageButton(base.icons.fg_vehicle_pigeon, icon_size) then
    MPVehicleGE.replaceRequest("autobello", { config = "turdie" })
  end

  im.Separator()
  im.Spacing()

  --[[ -- old picoturdie
  im.TextWrapped("Spawn a Boomer Pico (old players):")
  im.SameLine(im.GetContentRegionAvailWidth() - icon_size.x) -- right align
  if base.uiIconImageButton(base.icons.fg_vehicle_pigeon, icon_size) then
    MPVehicleGE.replaceRequest("autobello", {config = "turdieold.pc"})
  end

  im.Separator() im.Spacing()
  --]]

  drawSpawnLastUsedConfig()

  im.Separator()
  im.Separator()
  im.Spacing()

  drawSpawnCustomConfigs()

  base.endWindow()
end

-- draw functions <--

local function onExtensionLoaded()
  base.registerWindow("GarageUI", im.ImVec2(500, 200))
  base.registerWindow("CarpSpawningUI", im.ImVec2(450, 300)) -- spawning stuff, lets you spawn vehicles

  if not base.getUISetting('hide_window_CarpSpawningUI') then base.showWindow("CarpSpawningUI") end
  base.registerDrawItem("spawnLastUsedConfig", 'Spawn last used config', drawSpawnLastUsedConfig)
  base.registerDrawItem("spawnCustomConfigs", 'Spawn custom configs', drawSpawnCustomConfigs)

  loadSlotNames()

  print("Garage UI initialized.")
end

local function showGaragePopup(show, name)
  if show then
    UI_GARAGE_NAME = name
    base.showWindow("GarageUI")
  else
    base.hideWindow("GarageUI")
  end
end

local function showWindow(show)
  if show then
    base.showWindow("CarpSpawningUI")
  else
    base.hideWindow("CarpSpawningUI")
  end
end

local function onUpdate()
  drawGaragePopup()
  drawSpawningUI()
end

M.onExtensionLoaded = onExtensionLoaded
M.showGaragePopup = showGaragePopup
M.showWindow = showWindow
M.onUpdate = onUpdate

print("Garage UI loaded")
return M
