--====================================================================================
-- All work by Pastry and deer.
-- You have no permission to edit, redistribute or upload. That's all!
--====================================================================================
-- CaRP party UI
--====================================================================================
local M = {}
print("Loading Carp party UI...")

M.dependencies = {"ui_imgui", "carp_ui_base"}

local im = ui_imgui
local base = carp_ui_base

-- draw functions: -->
local function drawEmergencyRefuel()
  im.Columns(2, "carphelp")

  local icon_size = base.getIconSize()
  im.TextWrapped("Run out of fuel? Overpay for fuel here:")
  im.NextColumn()
  if base.uiIconImageButton(base.icons.fg_barrel, icon_size) then
    UI.chatSend("/refuel")
  end
  im.SameLine()
  im.TextWrapped("Emergency Refuel")

  im.Columns(1)
end

local function drawNearestLocationMenu()
  local icon_size = base.getIconSize()

  --im.Columns(2, "carpnearestlocations")

  -- dont show self-service GPS stuff if you have a mission
  if not carp.getCurrentMission().missionDestination and not carp_race.isInRace() then
    im.TextWrapped("Lost? CaRP Self-Service Location Services!")
    --im.NextColumn()

    local entries = {
      {key = "contact", title = "Contact", icon = "person_pin_circle"},
      {key = "garage", title = "Garage", icon = "build"},
      {key = "gas", title = "Gas", icon = "local_gas_station"},
      {key = "cardealership", title = "Dealership", icon = "fg_vehicle_sports_car"}
    }

    for _, entry in pairs(entries) do
      if base.uiIconImageButton(base.icons[entry.icon], icon_size) then
        carp.getNearestPOI(entry.key)
      end
      im.SameLine() --im.TextWrapped(entry.title) --simply using the 'label' parameter removes the icon bg
      if im.IsItemHovered() then
        im.SetTooltip(entry.title)
      end
    end

  --im.NextColumn()
  end

  --im.Columns(1)
end

local function drawNodegrabMenu()
  im.Columns(2, "carphelp")

  im.TextWrapped("Stuck or flipped over? Get help here!")
  im.NextColumn()

  if im.Button("Push it out yourself") then
    carp.enableNodegrab("yourself")
  end
  im.Spacing()

  if im.Button("Pay some locals") then
    carp.enableNodegrab("locals")
  end

  im.NextColumn()

  im.Separator()

  im.TextWrapped("Stuck in a ditch but don't want to lose your streak?")
  im.NextColumn()

  if im.Button("Tow to nearest road ($$$)") then
    carp_vehicle.towToNearestRoad()
  end

  im.NextColumn()

  im.Columns(1)
end

local function drawHelpUI()
  if not base.beginWindow("CarpHelpUI", "CaRP Help") then
    base.endWindow()
    return
  end

  im.SetWindowFontScale(base.getUISetting("window_scale"))
  local icon_size = base.getIconSize()

  im.TextWrapped("Welcome to CaRP!")
  im.TextWrapped("You can drag these windows onto one another to turn them into tabs!")

  im.Columns(2, "carphelp")

  im.TextWrapped("For help, tutorials and more, join our Discord:")
  im.NextColumn()
  if im.Button("Discord") then
    openWebBrowser("https://discord.com/invite/jjgeNTsVE3")
  end
  im.NextColumn()

  im.Columns(1)
  im.Separator()

  drawEmergencyRefuel()

  im.Separator()

  drawNodegrabMenu()

  im.Separator()

  drawNearestLocationMenu()

  if not carp.getCurrentMission().missionDestination and not carp_race.isInRace() then
    im.Separator()
  end

  im.Columns(2, "carpsettings")

  im.NextColumn()
  im.Separator()

  im.Columns(1)

  base.endWindow()
end
-- draw functions <--

local function onExtensionLoaded()
  if MPCoreNetwork then
    base.registerWindow("CarpHelpUI", im.ImVec2(450, 300), nil, false)

    base.registerDrawItem(
      "nearestLocations",
      "Nearest locations",
      drawNearestLocationMenu,
      function()
        return not carp.getCurrentMission().missionDestination and not carp_race.isInRace()
      end
    )
    base.registerDrawItem("emergencyRefuel", "Emergency refuel", drawEmergencyRefuel)
    base.registerDrawItem("nodegrabMenu", "Rescue system", drawNodegrabMenu)

    if not base.getUISetting("hide_window_CarpHelpUI") then
      base.showWindow("CarpHelpUI")
    end

    print("Carp help UI initialized.")
  end
end

local function showWindow(show)
  if show then
    base.showWindow("CarpHelpUI")
  else
    base.hideWindow("CarpHelpUI")
  end
end

local function onUpdate()
  drawHelpUI()
end

M.showWindow = showWindow
M.onExtensionLoaded = onExtensionLoaded
M.onUpdate = onUpdate

print("Carp UI loaded")
return M
