--====================================================================================
-- All work by Pastry and deer.
-- You have no permission to edit, redistribute or upload. That's all!
--====================================================================================
-- CaRP settings UI
--====================================================================================
local M = {}
print("Loading Carp settings UI...")

M.dependencies = { "ui_imgui", "carp_ui_base" }

local im = ui_imgui
local base = carp_ui_base

local icon_size = base.getIconSize()

local settings = {}
settings.volume = im.IntPtr(base.getUISetting("sfx_volume")) or im.IntPtr(5)
settings.distanceLights = im.IntPtr(base.getUISetting("distance_lights_distance")) or im.IntPtr(50)
settings.distanceLightsEnabled = im.BoolPtr(base.getUISetting("distance_lights_enabled")) or im.BoolPtr(true)
settings.groundMarkersDisabled = im.BoolPtr(base.getUISetting("disable_groundmarkers")) or im.BoolPtr(false)
settings.missionAutoFinishEnabled = im.BoolPtr(base.getUISetting("mission_auto_complete")) or im.BoolPtr(true)
settings.rainEnabled = im.BoolPtr(base.getUISetting("rain_enabled")) or im.BoolPtr(true)
settings.garageModeEnabled = im.BoolPtr(base.getUISetting("garage_mode_enabled")) or im.BoolPtr(false)

-- setting helper functions: -->
-- setting helper functions <--

-- draw functions: -->
local function drawPassengerVolumeSetting()
  im.Text("Passenger SFX Volume")

  if im.SliderInt("Volume", settings.volume, 0, 10) then
    base.saveUISetting("sfx_volume", settings.volume[0])
  end

  im.Text("")
end

local function drawDistanceLightsSetting()
  im.Text("Dynamic Distance Lights")

  if im.Checkbox("Enabled?", settings.distanceLightsEnabled) then
    base.saveUISetting("distance_lights_enabled", settings.distanceLightsEnabled[0])
  end

  if im.SliderInt("Distance", settings.distanceLights, 5, 500) then
    base.saveUISetting("distance_lights_distance", settings.distanceLights[0])
  end

  im.Text("")
end

local function drawRainSetting()
  im.Text("Raindrops Effect")

  if im.Checkbox("Enabled?", settings.rainEnabled) then
    base.saveUISetting("rain_enabled", settings.rainEnabled[0])
  end

  im.Text("")
end

local function drawOtherSettings()
  im.TextWrapped("Other settings:")

  if im.Checkbox("Hide groundmarkers?", settings.groundMarkersDisabled) then
    base.saveUISetting("disable_groundmarkers", settings.groundMarkersDisabled[0])
  end

  if im.Checkbox("Mission auto-finish", settings.missionAutoFinishEnabled) then
    base.saveUISetting("mission_auto_complete", settings.missionAutoFinishEnabled[0])
  end

  if im.Checkbox("Garage Mode (experimental; press F5)", settings.garageModeEnabled) then
    base.saveUISetting("garage_mode_enabled", settings.garageModeEnabled[0])
  end
end

local function drawSettingsUI()
  if not base.beginWindow("CarpSettingsUI", "CaRP Settings Menu") then
    base.endWindow()
    return
  end

  im.SetWindowFontScale(base.getUISetting("window_scale"))

  im.Columns(1, "Asd")

  im.TextWrapped("Settings:")
  im.Text("")

  drawPassengerVolumeSetting()
  im.Separator()
  drawDistanceLightsSetting()
  im.Separator()
  drawRainSetting()
  im.Separator()
  drawOtherSettings()

  base.endWindow()
end

-- draw functions <--

local function onExtensionLoaded()
  if MPCoreNetwork then
    base.registerWindow("CarpSettingsUI", im.ImVec2(450, 300)) -- settings!

    if not base.getUISetting("hide_window_CarpSettingsUI") then
      base.showWindow("CarpSettingsUI")
    end

    print("Carp settings UI initialized.")
  end
end

local function showWindow(show)
  if show then
    base.showWindow("CarpSettingsUI")
  else
    base.hideWindow("CarpSettingsUI")
  end
end

local function onUpdate()
  drawSettingsUI()
end

M.showWindow = showWindow
M.onExtensionLoaded = onExtensionLoaded
M.onUpdate = onUpdate

print("Carp UI loaded")
return M
