--====================================================================================
-- All work by Pastry and deer.
-- You have no permission to edit, redistribute or upload. That's all!
--====================================================================================
-- CaRP Weather UI
--====================================================================================
local M = {}
print("Loading Carp Weather UI...")

M.dependencies = {"ui_imgui", "carp_ui_base"}

local im = ui_imgui
local base = carp_ui_base

local headingFont = "cairo_semibold_large"
local icon_size = base.getIconSize()
local hideWindow = nil
local weatherData = nil

--

local messagesAndRanges = {
  cloudCover = {
    messages = {"very few clouds", "some clouds", "many clouds", "a completely overcast sky"},
    ranges = {10, 40, 80}
  },
  windSpeed = {
    messages = {"barely any wind", "breeze winds", "gale winds", "storm winds"},
    ranges = {3, 4, 5}
  },
  rainDrops = {
    messages = {"no rain", "some rain", "strong rain", "very heavy rain"},
    ranges = {50, 70, 80}
  },
  fogDensity = {
    messages = {"no fog", "light fog", "foggy conditions", "thick fog"},
    ranges = {15, 40, 80}
  },
  gravity = {
    messages = {"extremely .. light?", "somewhat .. light?", "somewhat .. heavy?", "extremely .. heavy?"},
    ranges = {15, 40, 80} -- TODO: FILL THESE OUT BEFORE YOU ENABLE GRAVITY
  }
}

local icons = {
  sun = "wb_sunny",
  cloud = "simobject_basic_clouds",
  verycloudy = "simobject_cloud_layer",
  rain = "invert_colors"
}

local componentNames = {"cloudCover", "windSpeed", "rainDrops", "fogDensity"}

local function getForecastComponent(component)
  local componentData = messagesAndRanges[component]
  for messageIndex, treshold in ipairs(componentData.ranges) do
    if weatherData[component] <= treshold then
      return componentData.messages[messageIndex]
    end
  end

  return componentData.messages[#componentData.messages] -- return last
end

local function getForecastMessage()
  local finalMessage = "Looks like there's"

  for _, component in ipairs(componentNames) do
    finalMessage = finalMessage .. " " .. getForecastComponent(component) .. ","
  end

  finalMessage = finalMessage:sub(1, -2) .. "."

  return finalMessage
end

local function getWeatherIcon()
  if not weatherData then
    return base.uiIconImage(base.icons.error_outline, icon_size)
  end

  if weatherData.rainDrops and weatherData.rainDrops >= 50 then
    return base.uiIconImage(base.icons[icons.rain], icon_size)
  end

  if weatherData.cloudCover and weatherData.cloudCover >= 40 then
    return base.uiIconImage(base.icons[icons.verycloudy], icon_size)
  end

  if weatherData.cloudCover and weatherData.cloudCover >= 10 then
    return base.uiIconImage(base.icons[icons.cloud], icon_size)
  end

  return base.uiIconImage(base.icons[icons.sun], icon_size)
end

local function renderMessageContents()
  if not weatherData then
    return
  end

  im.TextWrapped(getForecastMessage())
end

--=============== Trader Layout Functions ===============--
local function drawWeatherPopup()
  if not weatherData then
    return
  end

  if not base.beginWindow("CarpWeatherUI", "CaRP Weather UI", nil, true) then
    base.endWindow()
    return
  end

  im.SetWindowFontScale(base.getUISetting("window_scale"))

  local icon_size = base.getIconSize()

  im.Spacing()

  --
  getWeatherIcon()
  im.SameLine()
  renderMessageContents()

  base.endWindow()
end

--=============== Imgui Render Functions ===============--
hideWindow = function()
  weatherData = nil
  base.hideWindow("Weather")
end

local function onUpdate()
  drawWeatherPopup()
end

--=============== BeamMP Integration Functions ===============--
local function receiveWeatherFromServer(payload)
  weatherData = jsonDecode(payload)
end

local function onExtensionLoaded()
  base.registerWindow("CarpWeatherUI", im.ImVec2(500, 75))

  AddEventHandler("receiveWeatherFromServer", receiveWeatherFromServer)

  print("Weather UI initialized.")
end

M.onExtensionLoaded = onExtensionLoaded
M.showWindow = showWindow
M.onUpdate = onUpdate

print("Weather UI loaded")
return M
