local M = {}

local origOnVehicleSpawned = nil
local origSendDataToUI = nil
local bkpDropPlayer = nil
local jbeamIO = require("jbeam/io")
local jbeamSlotSystem = require('jbeam/slotSystem')
local vehManager = extensions.core_vehicle_manager

local lastResetPressed

local ownedParts = {}

local lastVehiclePosition = nil

currentVehicles = { car = {} }
local currentVehicleName
local firstSpawn = true

local timerDeltas = { queryPos = 0, queryFuel = 0, sendPos = 0 }
local updateRates = { queryPos = 1, queryFuel = 5, sendPos = 20 }

local pendingEditVeh = nil -- stores ID of last edited vehicle

local jbeamNames = {} -- used to figure out if we edited or spawned a new vehicle
local fuelMap = {}
local damageMap = {}
local oldDamageMap = {} -- TODO: probably the wrong way of doing things but at least i dont have to rewrite everything
local damageStats = {}

local partPrices = {}
local partPricesLoaded = false

local function loadPartPricesFromJson()
  if partPricesLoaded then
    return
  end

  partPricesLoaded = true

  local basePath = '/carpresources/json/priceCatalog'

  local partPriceFiles = FS:directoryList(basePath)

  if not #partPriceFiles or #partPriceFiles == 0 then
    log("W", "loadPartPricesFromJson", "No part prices found")
    return
  end

  local loadedPartCount = 0

  for _, filename in pairs(partPriceFiles) do
    local fileContents = jsonReadFile(filename)

    for partName, partPrice in pairs(fileContents) do
      partPrices[partName] = partPrice
      loadedPartCount = loadedPartCount + 1
      --table.insert(partPrices, partName, partPrice)
    end
  end

  log("I", "loadPartPricesFromJson", "Loaded " .. loadedPartCount .. " part prices")
end

-- function override for sending data to the part picker menu
local function customSendDataToUI()
  local playerVehicle = vehManager.getPlayerVehicleData()
  if not playerVehicle then
    log("E", "customSendDataToUI", "no active vehicle")
    return
  end

  local jbeam = stringToTable(playerVehicle.vehicleDirectory, '/')[2] -- '/vehicles/pickup/'

  local parts = jbeamIO.getAvailableParts(playerVehicle.ioCtx)

  loadPartPricesFromJson()

  local data = {
    mainPartName = playerVehicle.mainPartName,
    chosenParts = playerVehicle.chosenParts,
    variables = playerVehicle.vdata.variables,
    slotMap = jbeamIO.getAvailableSlotMap(playerVehicle.ioCtx)
  }

  data.availableParts = shallowcopy(parts)

  -- enrich the data a bit for the UI
  for partName, part in pairs(data.availableParts) do
    if part.modName then
      local mod = core_modmanager.getModDB(part.modName)
      if mod and mod.modData then
        part.modTagLine = mod.modData.tag_line
        part.modTitle = mod.modData.title
        part.modLastUpdate = mod.modData.last_update
      end
    end
    if ownedParts[jbeam] and ownedParts[jbeam][partName] then -- from server
      part.value = "(Owned)"
    elseif partPrices[partName] then -- from local json
      part.value = "($" .. tostring(partPrices[partName]) .. ")"
    elseif type(part.value) == "number" then -- fallback, from jbeam
      part.value = "(" .. tostring(part.value) .. ")"
    end
  end

  guihooks.trigger("VehicleConfigChange", data)
  guihooks.trigger("VehiclePartsTree", data)
end

local function customOnVehicleSpawned(gameVehicleID)
  local vehicleData = vehManager.getVehicleData(gameVehicleID)

  if not vehicleData or not MPVehicleGE.isOwn(gameVehicleID) then origOnVehicleSpawned(gameVehicleID) return end

  local vehicle, unifyJournal, chosenParts, activePartsOrig = jbeamSlotSystem.findParts(vehicleData.ioCtx,
    vehicleData.config)

  chosenParts.main = nil

  vehicleData.config.parts = chosenParts -- this updates the config for future calls of getVehicleData too, so edits are also taken care of

  origOnVehicleSpawned(gameVehicleID)
end

local function injectPartPrices()
  if extensions.core_vehicle_partmgmt and extensions.core_vehicle_partmgmt.sendDataToUI ~= customSendDataToUI then
    log("E", "onclientpoststartmission", "loaded custom part schenanigans")
    origSendDataToUI = extensions.core_vehicle_partmgmt.sendDataToUI
    extensions.core_vehicle_partmgmt.sendDataToUI = customSendDataToUI
    extensions.core_vehicle_partmgmt.reset = customSendDataToUI
  end
end

local function injectConfigCleanup()
  if extensions.MPVehicleGE and extensions.MPVehicleGE.onVehicleSpawned ~= customOnVehicleSpawned then
    log("E", "onExtensionLoaded", "loaded custom config schenanigans")
    origOnVehicleSpawned = extensions.MPVehicleGE.onVehicleSpawned
    extensions.MPVehicleGE.onVehicleSpawned = customOnVehicleSpawned

    extensions.hookUpdate("onVehicleSpawned") -- its a hook so we have to invalidate the cache
  end
end

-- return local vehicle object
local function getPlayerVehicle()
  if currentVehicles.car and currentVehicles.car.gameVehicleID then
    local veh = be:getObjectByID(currentVehicles.car.gameVehicleID)
    if veh then
      return veh
    end
  end

  log("E", "getPlayerVehicle", "Could not get carp current veh, falling back to game current vehicle")
  local veh = be:getPlayerVehicle(0)

  if veh and MPVehicleGE.isOwn(veh:getID()) then
    currentVehicles.car.gameVehicleID = veh:getID()
    return veh
  end

  return nil
end

-- return local vehicle id
local function getPlayerVehicleID()
  if not currentVehicles.car.gameVehicleID then
    log("E", "getPlayerVehicleID", "Could not get currentVehicles.car.gameVehicleID, falling back to current vehicle")
    local currveh = getPlayerVehicle()
    if not currveh then
      log("E", "getPlayerVehicleID", "Could not get be:getPlayerVehicle(0)")
      return nil
    end
    currentVehicles.car.gameVehicleID = currveh:getID()
  end

  return currentVehicles.car.gameVehicleID
end

-- return local trailer id
local function getTrailerVehicleID()
  if currentVehicles.trailer and currentVehicles.trailer.gameVehicleID then
    return currentVehicles.trailer.gameVehicleID
  else
    log("E", "getTrailerVehicleID", "Could not get currentVehicles.trailer.gameVehicleID")
    return nil
  end
end

-- return local vehicle position object
local function getVehiclePosition()
  local playerVehicle = getPlayerVehicle()
  if playerVehicle then
    local playerVehiclePosition = playerVehicle:getPosition()

    lastVehiclePosition = playerVehiclePosition
    return playerVehiclePosition
  end
end

-- used when 'teleporting'
local function sendPositionDifference()
  if not lastVehiclePosition or not lastVehiclePosition.x then
    return false
  end

  local lastVehPos = vec3(lastVehiclePosition)

  local currentPos = getVehiclePosition()
  if not currentPos then
    return false
  end

  local dist = currentPos:distance(lastVehPos)

  TriggerServerEvent("receiveHandleTeleport", tostring(dist))
end

local function getVehicleName()
  local vehic = getPlayerVehicle()
  if vehic and vehic.JBeam then
    return vehic.JBeam
  end
end

-- updates fuel info (from VE)
local function setFuelInfo(vehID, fuelCapacity, fuelRemaining, nitrousCapacity, nitrousRemaining)
  fuelMap[vehID] = {
    capacity = fuelCapacity,
    current = fuelRemaining,
    nitrousCapacity = nitrousCapacity,
    nitrousRemaining = nitrousRemaining
  }
end

-- request fuel from VE
local function getFuelInfo()
  local currVeh = getPlayerVehicle()
  if currVeh then
    currVeh:queueLuaCommand("if carp then carp.sendFuel() end")
  end
end

local function receiveRefuelCommand(payload)
  local payloadSplit = stringToTable(payload, "|")

  local vehicleName, currentFuel, currentNitrous = payloadSplit[1], payloadSplit[2], payloadSplit[3]

  local veh = getPlayerVehicle()
  if not veh or not veh:getID() then
    return
  end

  veh:queueLuaCommand(
    "local s = energyStorage.getStorages() local f = 0 if (s) then for k, v in pairs(s) do if (k ~= 'mainBottle') then f = f + v.capacity end end end  "
    ..
    "local fuelRatio = " ..
    currentFuel ..
    " / f " .. "for k, v in pairs(s) do " .. "if (k ~= 'mainBottle') then v:setRemainingRatio(fuelRatio) end end "-- ..
  )

  veh:queueLuaCommand(
    "local s = energyStorage.getStorages() local f = 0 if (s) then for k, v in pairs(s) do if (k == 'mainBottle') then f = f + v.capacity end end end "
    ..
    "local fuelRatio = " ..
    currentNitrous ..
    " / f " ..
    "for k, v in pairs(s) do " ..
    "if (k == 'mainBottle' and v.setRemainingRatio) then v:setRemainingRatio(fuelRatio) end end "-- ..
  )

  if not fuelMap[veh:getID()] then
    return
  end

  fuelMap[veh:getID()].current = currentFuel
end

-- updates damage stats (from VE)
local function setDamageStats(vehID, beams_broken, beams_deformed, beam_count)
  damageMap[vehID] = {
    brokenBeams = beams_broken,
    deformedBeams = beams_deformed,
    totalBeams = beam_count
  }
end

-- returns damage stats for local vehicle
local function getDamageStats()
  local vehID = getPlayerVehicleID()
  return vehID and damageMap[vehID] or nil
end

-- TODO: cleanup. why are there 2 similar functions
local function receiveChatRefuel()
  local playerVehicleID = getPlayerVehicleID()
  if not fuelMap[playerVehicleID] then
    getFuelInfo()
    return
  else
    local payload = getVehicleName() .. "|" .. fuelMap[playerVehicleID].current
    TriggerServerEvent("receiveChatRefuelResult", payload)
  end
end

local function sendFuel()
  local vehicleName = getVehicleName()
  local playerVehicleID = getPlayerVehicleID()

  --check if we know the current fuel levels for this vehicle. if not, get them
  if not vehicleName or not fuelMap[playerVehicleID] then
    getFuelInfo()
    return
  end

  local payload =
  vehicleName ..
      "|" .. tostring(fuelMap[playerVehicleID].current) .. "|" .. tostring(fuelMap[playerVehicleID].nitrousRemaining)
  TriggerServerEvent("receiveUpdateVehicleFuel", payload)
end

-- UI button
local function requestRefuel()
  local playerVehicleID = getPlayerVehicleID()
  if not playerVehicleID or not fuelMap[playerVehicleID] then
    getFuelInfo()
    return
  else
    local payload =
    getVehicleName() .. "|" .. fuelMap[playerVehicleID].capacity .. "|" .. fuelMap[playerVehicleID].current
    TriggerServerEvent("receiveRefuelVehicleStation", payload)
  end
end

-- UI button
local function requestNitrousRefuel()
  local playerVehicleID = getPlayerVehicleID()
  if not playerVehicleID or not fuelMap[playerVehicleID] then
    getFuelInfo()
    return
  else
    local payload =
    getVehicleName() ..
        "|" .. fuelMap[playerVehicleID].nitrousCapacity .. "|" .. fuelMap[playerVehicleID].nitrousRemaining
    TriggerServerEvent("receiveNitrousRefuelVehicleStation", payload)
  end
end

-- returns a string with: position if no damage stats
-- returns a string with: position + brokenBeams + deformedBeams + vehicleName, if we have em
local function buildVehicleResettedPayload()
  local playerVehicle = getPlayerVehicle()
  local playerVehicleID = playerVehicle:getID()
  local playerVehiclePosition = playerVehicle:getPosition()

  local positionString = playerVehiclePosition.x .. "&" .. playerVehiclePosition.y .. "&" .. playerVehiclePosition.z
  local damageStats = damageMap[playerVehicleID]
  local lastRepairDamageStats = oldDamageMap[playerVehicleID]

  dump(damageMap)
  dump(damageStats)

  if damageStats ~= nil and damageStats.brokenBeams ~= nil then
    local returnString = ""

    -- if everything is the same as the last repair, send that the vehicle is in perfect shape instead
    if lastRepairDamageStats and tableSize(MPHelpers.tableDiff(damageStats, lastRepairDamageStats)) == 0 then
      returnString = positionString .. "|0|0|3000"
    else
      returnString =
      positionString ..
          "|" .. damageStats.brokenBeams .. "|" .. damageStats.deformedBeams .. "|" .. damageStats.totalBeams
    end

    returnString = returnString .. "|" .. currentVehicleName

    -- reset the last repaired values
    oldDamageMap[playerVehicleID] = shallowcopy(damageStats) -- bruh moment

    return returnString
  else
    return positionString
  end
end

local function checkDamagedVehicleChange(previousVehicleName, vehicleName)
  if previousVehicleName == vehicleName or previousVehicleName == nil or vehicleName == nil then
    return
  end

  if damageStats["brokenBeams"] and damageStats["deformedBeams"] and damageStats["totalBeams"] and
      tostring(lastRepairDamageStats["brokenBeams"]) ~= tostring(damageStats["brokenBeams"]) and
      tostring(lastRepairDamageStats["deformedBeams"]) ~= tostring(damageStats["deformedBeams"]) and
      tostring(lastRepairDamageStats["totalBeams"]) ~= tostring(damageStats["totalBeams"])
  then
    local payloadString = buildVehicleResettedPayload()
    payloadString = payloadString .. "|" .. previousVehicleName
    TriggerServerEvent("receiveHandlePlayerSwapDamagedVehicle", payloadString)

    damageStats = {}
  end
end

local function towToNearestRoad()
  local playerVehicle = getPlayerVehicle()

  if playerVehicle:getVelocity():length() > 5 then
    TriggerServerEvent("genericMessageSender", "Slow down there cowboy")
    return
  end

  local nearestRoadNode, distance = carp_groundmarkers.findClosestRoad(playerVehicle:getPosition())

  if not distance or distance > 25 then
    TriggerServerEvent(
      "genericMessageSender",
      "The towing service had a good look at your wreck and decided you're too far to pull out."
    )
    return
  end

  local nearestRoadPos = vec3(map.getMap().nodes[nearestRoadNode].pos)
  nearestRoadPos.z = nearestRoadPos.z + 0.1

  playerVehicle:setPositionNoPhysicsReset(nearestRoadPos)
  commands.setGameCamera()

  TriggerServerEvent("receiveHandleTow", tostring(distance))
end

local function removeTrailerVehicle()
  local trailerGameVehicleID = getTrailerVehicleID()

  if carp.hasTrailerMission() and trailerGameVehicleID then
    local trailerObj = be:getObjectByID(trailerGameVehicleID)
    if trailerObj then
      trailerObj:delete()
    end
    currentVehicles.trailer = nil
  else
  end
end

local function exitedGarage()
end

-- SERVER EVENTS

-- receives owned parts so it can be injected into the UI
local function receiveOwnedParts(partstring)
  local ownedPartsArray = stringToTable(partstring, ";")
  local vehicleName = ownedPartsArray[1]
  log("I", "receiveOwnedParts", "Received " .. #ownedPartsArray .. " owned parts")
  ownedParts[vehicleName] = {} -- clear previous entries for this vehicle
  for k, v in pairs(ownedPartsArray) do
    ownedParts[vehicleName][v] = true
  end
end

local function setPlayerVehicleSlot(data)
  local decodedData = stringToTable(data, "|")

  log("W", "setPlayerVehicleSlot", dumps(decodedData))

  local vehicleType = decodedData[1]
  local vehicleName = decodedData[2]
  local gameVehicleID = tonumber(decodedData[4])

  -- .... I'm not so sure about this? what is this for..
  --local serverVehicleID = MPConfig.getPlayerServerID() .. "-" .. decodedData[3]

  if vehicleType then
    if not currentVehicles[vehicleType] then
      currentVehicles[vehicleType] = {}
    end
    currentVehicles[vehicleType].serverVehicleID = serverVehicleID

    if gameVehicleID and gameVehicleID > 0 then
      currentVehicles[vehicleType].gameVehicleID = gameVehicleID --MPVehicleGE.getGameVehicleID(serverVehicleID)
    end

    if currentVehicles[vehicleType].name == vehicleName then
      if currentVehicles[vehicleType].spawnCount then
        currentVehicles[vehicleType].spawnCount = 1
      end
    else
      currentVehicles[vehicleType].name = vehicleName

      if not currentVehicles[vehicleType].spawnCount then
        currentVehicles[vehicleType].spawnCount = 0
      end
    end
  end
end

local function spawnConfig(data)
  if data == "picoturdie" or data == '"picoturdie"' then
    MPVehicleGE.replaceRequest("autobello", { config = "turdie" })
    return
  else
    local config = jsonDecode(data)

    local requestConfig = {
      config = config.vcf
    }
    if config.rot and config.rot.x and config.rot.y and config.rot.z and config.rot.w then
      requestConfig.rot = quat(config.rot) * quat(0, 0, 1, 0) -- rotate vehicle 180 degrees
    else
      log("W", "spawnConfig", "Vehicle spawn rotation data unvailable.")
    end

    if config.pos and config.pos.x and config.pos.y and config.pos.z then
      requestConfig.pos = config.pos
    else
      log("W", "spawnConfig", "Vehicle spawn position data unvailable. Spawning at random default spawn location.")
      local spawnLocations = scenetree.findClassObjects("SpawnSphere")
      local totalSpawns = 0

      for _, name in pairs(spawnLocations) do
        -- literally just looping to figure out how many :(
        totalSpawns = totalSpawns + 1
      end

      if (totalSpawns > 0) then
        math.randomseed(os.time())
        local spawnToUse = math.floor((math.random() * totalSpawns) + 1)
        local spawnObject = scenetree.findObject(spawnLocations[spawnToUse])
        requestConfig.pos = spawnObject:getPosition()
      end
    end

    if requestConfig.pos and requestConfig.pos.x == 0 and requestConfig.pos.y == 0 and requestConfig.pos.z == 0 then
      TriggerServerEvent("genericMessageSender", "Could not load your position! This is bad!")
      return
    end

    --replace current vehicle if owned
    MPVehicleGE.replaceRequest(config.jbm, requestConfig)
  end
end

local function spawnLastPlayerVehicle()
  TriggerServerEvent("receiveLastLocationAndVehicle", "a")
end

-- BEAMNG EVENTS

local function onVehicleResetted(vehID, other)
  if not MPVehicleGE or not MPVehicleGE.isOwn(vehID) or not MPVehicleGE.getServerVehicleID(vehID) then
    return
  end

  MPVehicleGE.sendVehicleEdit(vehID)

  log("W", "onVehicleResetted", "" .. vehID)

  getFuelInfo()

  sendPositionDifference()

  local previousVehicleName = currentVehicleName
  currentVehicleName = getVehicleName()
  log("W", "onVehicleResetted", "2")
  local currentMission = carp.getCurrentMission()

  if (currentMission ~= nil and MPVehicleGE and MPVehicleGE.isOwn(vehID)) then
    if ((currentVehicleName ~= previousVehicleName) and currentMission.missionDestination ~= nil) then
      removeTrailerVehicle()
      UI.chatSend("/abandon")
    end
  end
  log("W", "onVehicleResetted", "3")

  -- lets delete trailers if you give up on a special mission!
  if carp.hasTrailerMission() then
    if currentVehicles.trailer and currentVehicles.trailer.spawnCount and currentVehicles.trailer.spawnCount < 2 then
      currentVehicles.trailer.spawnCount = currentVehicles.trailer.spawnCount + 1
      return
    else
      removeTrailerVehicle()
      TriggerServerEvent("receiveHandleSpecialMissionAttemptedReset", tostring(vehID))
    end
  end
  log("W", "onVehicleResetted", "4")

  if firstSpawn then
    firstSpawn = false

    MPModManager.checkAllMods()
  else
    local payloadString = buildVehicleResettedPayload()
    log("W", "onVehicleResetted", payloadString)

    TriggerServerEvent("receiveHandlePlayerResetVehicle", payloadString)
    getPlayerVehicle():queueLuaCommand("carp_dynamicWeight.setNodeWeights()") -- set node weights back to what they should be on resets
  end

  -- damaged vehicle checks
  local vehName = getVehicleName()
  checkDamagedVehicleChange(previousVehicleName, vehName)

  TriggerServerEvent("receiveGetVehicleFuelAmount", vehName)

  --getMaxFuel()
end

local function onVehicleDestroyed(vehID)
  local mission = carp.getCurrentMission()
  if (mission ~= nil and MPVehicleGE and MPVehicleGE.isOwn(vehID) and currentVehicles.trailer == nil) then
    if (mission.missionDestination ~= nil) then
      removeTrailerVehicle()
      UI.chatSend("/abandon")
      --currentMission = {} the server should clear the mission for us
    end
  end
end

-- sync on part change
local function onVehicleSpawned(gameVehicleID)
  local veh = be:getObjectByID(gameVehicleID)
  local newJbeam = veh:getJBeamFilename()

  local oldJbeam = jbeamNames[gameVehicleID]

  if oldJbeam and oldJbeam == newJbeam then -- If it's an edit, not new spawn or vehicle change
    pendingEditVeh = gameVehicleID
  end

  jbeamNames[gameVehicleID] = newJbeam
end

local function onUpdate(dtReal, dtSim, dtRaw)
  for k, v in pairs(timerDeltas) do
    timerDeltas[k] = timerDeltas[k] + dtReal
  end

  -- used for caching in case of resets/teleports
  if timerDeltas.queryPos > updateRates.queryPos then
    timerDeltas.queryPos = 0
    --getDamageStats() now done on every frame
    getVehiclePosition()
  end

  if timerDeltas.queryFuel > updateRates.queryFuel then
    timerDeltas.queryFuel = 0
    getFuelInfo()
  end

  if timerDeltas.sendPos > updateRates.sendPos then
    timerDeltas.sendPos = 0

    sendFuel()
  end
end

local function onExtensionLoaded()
  if MPGameNetwork then
    AddEventHandler("onClientReceiveOwnedParts", receiveOwnedParts)
    AddEventHandler("clientCommandReceiver", sendMissionRequest)
    AddEventHandler("setPlayerVehicleSlot", setPlayerVehicleSlot)
    AddEventHandler("receiveRefuelCommand", receiveRefuelCommand)
    AddEventHandler("receiveChatRefuel", receiveChatRefuel)
    AddEventHandler("spawnConfig", spawnConfig)
    AddEventHandler("spawnLastPlayerVehicle", spawnLastPlayerVehicle)
  end

  resetGameplay = function(playerID)
    if lastResetPressed and (os.clock() - lastResetPressed) < 2 then
      ui_message("Commencing towing...", 5, "confirmTow")
      extensions.hook("onResetGameplay", playerID)
    else
      ui_message("Reset pressed once, press again to confirm towing", 5, "confirmTow")
    end
    lastResetPressed = os.clock()
  end

  bkpDropPlayer = commands.dropPlayerAtCamera
  commands.dropPlayerAtCamera = function()
    return nil
  end

  injectConfigCleanup()
  injectPartPrices()
end

local function onExtensionUnloaded()
  if origSendDataToUI ~= nil then
    extensions.core_vehicle_partmgmt.sendDataToUI = origSendDataToUI
  end

  commands.dropPlayerAtCamera = bkpDropPlayer
end

local function onWorldReadyState(state)
  if state == 2 then
    injectPartPrices()
  end
end

M.onVehicleResetted = onVehicleResetted
M.onVehicleDestroyed = onVehicleDestroyed
M.onVehicleSpawned = onVehicleSpawned
M.onUpdate = onUpdate
M.onExtensionLoaded = onExtensionLoaded
M.onExtensionUnloaded = onExtensionUnloaded
M.onWorldReadyState = onWorldReadyState

M.exitedGarage = exitedGarage
M.setFuelInfo = setFuelInfo
M.setDamageStats = setDamageStats
M.sendFuel = sendFuel
M.requestRefuel = requestRefuel
M.requestNitrousRefuel = requestNitrousRefuel
M.removeTrailerVehicle = removeTrailerVehicle
M.towToNearestRoad = towToNearestRoad

M.getPlayerVehicle = getPlayerVehicle
M.getPlayerVehicleID = getPlayerVehicleID
M.getVehiclePosition = getVehiclePosition
M.getTrailerVehicleID = getTrailerVehicleID

return M
